import streamlit as st
import pandas as pd
import numpy as np
import subprocess
import glob, sys, time, statistics, os.path, random
from pathlib import Path
import psutil
import plotly.graph_objects as go
from taxontabletools2.start import TTT_variables
from taxontabletools2.utilities import load_df
from taxontabletools2.utilities import collect_replicates
from taxontabletools2.utilities import collect_sample_stats
from taxontabletools2.basic_stats import basic_stats_reads
from taxontabletools2.basic_stats import basic_stats_OTUs
from taxontabletools2.basic_stats import taxonomic_resolution
from taxontabletools2.basic_stats import taxonomic_richness
from taxontabletools2.utilities import collect_traits
from taxontabletools2.utilities import strip_traits
from taxontabletools2.utilities import export_plot
from taxontabletools2.table_processing import replicate_merging
from taxontabletools2.table_processing import negative_control_subtraction
from taxontabletools2.table_processing import read_based_filter
from taxontabletools2.table_processing import read_based_normalisation
from taxontabletools2.table_processing import taxonomic_filtering
from taxontabletools2.table_processing import sample_filtering
from taxontabletools2.sample_comparison import venn
from taxontabletools2.read_analyses import read_props_bar
from taxontabletools2.read_analyses import read_props_heatmap
import streamlit as st
from taxontabletools2.start import start

# Call the sidebar function
settings = start()
users_settings = settings['new_user_preferences_dict']

## Page title
st.write(""" ## Read proportions """)

# Ensure the session state variables are retained across subpages
if 'TaXon_table_df' not in st.session_state:
    error1, error2, error3 = st.columns(3)
    with error2:
        st.write('##### Please load a TaXon table to continue!')

elif 'TaXon_table_xlsx' not in st.session_state:
    error1, error2, error3 = st.columns(3)
    with error2:
        st.write('##### Please load a TaXon table to continue!')

elif st.session_state['TaXon_table_df'].empty == True:
    error1, error2, error3 = st.columns(3)
    with error2:
        st.write('##### Please check your TaXon table. It seems to be empty!')

elif st.session_state['TaXon_table_xlsx'].is_file() == False:
    error1, error2, error3 = st.columns(3)
    with error2:
        st.write('##### The TaXon table does not exist in the selected project folder!')

########################################################################################################################

else:
    st.write('### Read proportion analyses')

    st.write('### Bar chart')

    reads1, reads2 = st.columns(2)

    with reads1:
        available_metadata = ['All samples']  + st.session_state['metadata_df'].columns.tolist()[1:]
        selected_metadata = st.selectbox('Select metadata to test:', available_metadata)

    with reads2:
        taxonomic_level = st.selectbox('Select taxonomic level:', st.session_state['available_taxonomic_levels_list'], index=5)

    if st.button(f'Calculate {taxonomic_level} reads proportions for {selected_metadata}', key='readprops_bar'):

        fig_read_props_1 = read_props_bar(
            st.session_state['path_to_outdirs'],
            st.session_state['TaXon_table_xlsx'],
            st.session_state['TaXon_table_df'],
            st.session_state['samples'],
            st.session_state['metadata_df'],
            selected_metadata,
            st.session_state['traits_df'],
            '',
            users_settings,
            {'selected_metadata': selected_metadata, 'taxonomic_level': taxonomic_level}
        )

        st.plotly_chart(fig_read_props_1, use_container_width=True, config=st.session_state['config'])
        export_plot(st.session_state['TaXon_table_xlsx'], 'Read_proportions_plots', 'bar_chart', selected_metadata, fig_read_props_1, users_settings, 'plotly')

    st.write('### Heatmap')

    reads3, reads4 = st.columns(2)

    with reads3:
        available_metadata = ['All samples']  + st.session_state['metadata_df'].columns.tolist()[1:]
        selected_metadata = st.selectbox('Select metadata to test: ', available_metadata)

    with reads4:
        taxonomic_level = st.selectbox('Select taxonomic level: ', st.session_state['available_taxonomic_levels_list'], index=5)

    if st.button(f'Calculate {taxonomic_level} reads proportions for {selected_metadata}', key='readprops_heatmap'):

        fig_read_props_1 = read_props_heatmap(
            st.session_state['path_to_outdirs'],
            st.session_state['TaXon_table_xlsx'],
            st.session_state['TaXon_table_df'],
            st.session_state['samples'],
            st.session_state['metadata_df'],
            selected_metadata,
            st.session_state['traits_df'],
            '',
            users_settings,
            {'selected_metadata': selected_metadata, 'taxonomic_level': taxonomic_level}
        )

        st.plotly_chart(fig_read_props_1, use_container_width=True, config=st.session_state['config'])
        export_plot(st.session_state['TaXon_table_xlsx'], 'Read_proportions_plots', 'heatmap', selected_metadata, fig_read_props_1, users_settings, 'plotly')


import math
import time
import pandas as pd
import numpy as np
from pathlib import Path
from taxontabletools2.utilities import add_traits, filter_taxontable, export_taxon_table, update_taxon_table
import plotly.graph_objects as go
import plotly.express as px
from scipy.spatial.distance import pdist, squareform
from scipy.spatial import ConvexHull
import streamlit as st
import plotly.colors
from taxontabletools2.utilities import load_df, collect_traits, strip_traits
import os, subprocess
from playwright.sync_api import sync_playwright

def convert_to_perlodes(path_to_outdirs, taxon_table_xlsx, taxon_table_df, samples, metadata_df, traits_df, tool_settings):
    # Make copies of input dataframes to prevent altering the originals
    taxon_table_df = taxon_table_df.copy()
    metadata_df = metadata_df.copy()
    river_types_dict = {i:j for i,j in metadata_df[['Sample', 'Perlodes_river_type']].values.tolist()}
    taxa_list_dict = {i:j for i,j in metadata_df[['Sample', 'Perlodes_taxa_list']].values.tolist()}
    usage_dict = {i:j for i,j in metadata_df[['Sample', 'Perlodes_usage']].values.tolist()}

    # Ensure all samples have metadata in all three categories
    samples = [
        i for i in samples
        if i in river_types_dict and pd.notna(river_types_dict[i]) and
           i in taxa_list_dict and pd.notna(taxa_list_dict[i]) and
           i in usage_dict and pd.notna(usage_dict[i])
            ]

    if len(samples) == 0:
        st.warning('Please fill out the required metadata for at least one sample!')
        return

    # Extract relevant settings from the tool settings
    presence_absence = tool_settings['presence_absence']
    taxon_table_taxonomy = taxon_table_df.columns.tolist()[1:7]

    all_reads = []
    for sample in samples:
        total_reads = taxon_table_df[sample].sum()
        taxon_dict = {}
        for OTU in taxon_table_df[taxon_table_taxonomy + [sample]].values.tolist():
            taxon = [item for item in OTU[0:6] if item != ''][-1]
            if taxon not in taxon_dict.keys():
                n_reads = OTU[-1]
                taxon_dict[taxon] = n_reads
            else:
                n_reads = OTU[-1]
                taxon_dict[taxon] = taxon_dict[taxon] + n_reads
        if presence_absence == True:
            pa_list = [1 if i != 0 else 0 for i in taxon_dict.values()]
            all_reads.append(pa_list)
        elif presence_absence == 'Relative':
            rel_list = [round(i / total_reads * 100, 2) for i in taxon_dict.values()]
            all_reads.append(rel_list)
        else:
            all_reads.append(list(taxon_dict.values()))

    # create initial df
    index = [i for i in range(1,1+len(taxon_dict.keys()))]
    species = list(taxon_dict.keys())
    perlodes_df = pd.DataFrame(species, columns=['species'])
    perlodes_df.insert(0, '', index)
    perlodes_df = pd.concat([perlodes_df, pd.DataFrame(all_reads, index=samples).transpose()], axis=1)

    # convert the df to match the (complicated) perlodes input format
    columns = ['ID_ART', 'TAXON_NAME'] + samples
    gewässertyp = ['Gewässertyp', ''] + [river_types_dict[i] for i in samples]
    Taxaliste = ['Taxaliste', ''] + [taxa_list_dict[i] for i in samples]
    Nutzung = ['Nutzung', ''] + [usage_dict[i] for i in samples]
    taxa = [[1] + i[1:] for i in perlodes_df.values.tolist()]
    df_list = [gewässertyp] + [Taxaliste] + [Nutzung] + taxa
    perlodes_input_df = pd.DataFrame(df_list, columns=columns)

    # write the filtered list to a dataframe
    if presence_absence == True:
        perlodes_directory = Path(str(path_to_outdirs) + "/" + "Perlodes" + "/" + taxon_table_xlsx.stem)
        perlodes_xlsx = Path(str(perlodes_directory) + "_Perlodes_PA.xlsx")
        perlodes_input_df.to_excel(perlodes_xlsx, index=False)
    elif presence_absence == 'Relative':
        perlodes_directory = Path(str(path_to_outdirs) + "/" + "Perlodes" + "/" + taxon_table_xlsx.stem)
        perlodes_xlsx = Path(str(perlodes_directory) + "_Perlodes_RELATIVE_ABUNDANCE.xlsx")
        perlodes_input_df.to_excel(perlodes_xlsx, index=False)
    else:
        perlodes_directory = Path(str(path_to_outdirs) + "/" + "Perlodes" + "/" + taxon_table_xlsx.stem)
        perlodes_xlsx = Path(str(perlodes_directory) + "_Perlodes_ABUNDANCE.xlsx")
        perlodes_input_df.to_excel(perlodes_xlsx, index=False)

    st.success(f'Wrote Perlodes input file to: {perlodes_xlsx}')

def convert_to_phylib(path_to_outdirs, taxon_table_xlsx, taxon_table_df, samples, metadata_df, traits_df, tool_settings):

    #get the taxonomy from the operational taxon list
    operational_taxon_list_df = pd.read_excel(Path(operational_taxon_list), sheet_name="TTT import").fillna('')

    # load the taxon table and create a list
    TaXon_table_xlsx = Path(TaXon_table_xlsx)
    TaXon_table_df = pd.read_excel(TaXon_table_xlsx).fillna('')
    meta_data_df = collect_metadata(TaXon_table_df)
    TaXon_table_df = strip_metadata(TaXon_table_df)
    TaXon_table_taxonomy = TaXon_table_df.columns.tolist()[0:7]
    samples_list = TaXon_table_df.columns.tolist()[10:]

    ## load the metadata -> freshwater type
    Meta_data_table_xlsx = Path(str(path_to_outdirs) + "/" + "Meta_data_table" + "/" + TaXon_table_xlsx.stem + "_metadata.xlsx")
    Meta_data_table_df = pd.read_excel(Meta_data_table_xlsx, header=0).fillna("")
    Meta_data_table_samples = Meta_data_table_df['Samples'].tolist()
    metadata_loc = Meta_data_table_df.columns.tolist().index(meta_data_to_test)
    types_dict = {i[0]:i[1] for i in Meta_data_table_df[['Samples', meta_data_to_test]].values.tolist()}

    ## drop samples with metadata called nan (= empty)
    drop_samples = [i[0] for i in Meta_data_table_df.values.tolist() if i[metadata_loc] == ""]

    ## test if samples have metadata
    if drop_samples != []:
        sg.PopupError("Please fill out all the metadata for all samples.")

    ## test if all metadata for Phylib is available
    elif len(set([True if i in Meta_data_table_df.columns.tolist() else False for i in ['Ökoregion', 'Makrophytenverödung', 'Begründung', 'Helophytendominanz', 'Diatomeentyp', 'Phytobenthostyp', 'Makrophytentyp', 'WRRL-Typ', 'Gesamtdeckungsgrad']])) != 1:
        sg.PopupError("Please fill out all the required phylib metadata for all samples.")

    elif sorted(Meta_data_table_samples) == sorted(samples_list):
        # store hits and dropped OTUs
        hit_list, dropped_list = [], []

        # loop through the taxon table
        for taxonomy in TaXon_table_df[TaXon_table_taxonomy].values.tolist():
            ## test species
            if taxonomy[6] != '' and taxonomy[6] in operational_taxon_list_df['Species'].values.tolist():
                ## merge the OTU's taxonomy with OTL information
                res = taxonomy + operational_taxon_list_df[operational_taxon_list_df['Species'].str.contains(taxonomy[6])][['DV-NR.', 'Taxon']].values.tolist()[0]

            ## test genus
            elif taxonomy[5] != '' and taxonomy[5] in operational_taxon_list_df['Genus'].values.tolist():
                ## merge the OTU's taxonomy with OTL information
                res = taxonomy + operational_taxon_list_df[operational_taxon_list_df['Genus'].str.contains(taxonomy[5])][['DV-NR.', 'Taxon']].values.tolist()[0]

            ## test family
            elif taxonomy[4] != '' and taxonomy[4] in operational_taxon_list_df['Family'].values.tolist():
                ## merge the OTU's taxonomy with OTL information
                res = taxonomy + operational_taxon_list_df[operational_taxon_list_df['Family'].str.contains(taxonomy[4])][['DV-NR.', 'Taxon']].values.tolist()[0]

            ## test order
            elif taxonomy[3] != '' and taxonomy[3] in operational_taxon_list_df['Order'].values.tolist():
                ## merge the OTU's taxonomy with OTL information
                res = taxonomy + operational_taxon_list_df[operational_taxon_list_df['Order'].str.contains(taxonomy[3])][['DV-NR.', 'Taxon']].values.tolist()[0]

            ## test class
            elif taxonomy[2] != '' and taxonomy[2] in operational_taxon_list_df['Class'].values.tolist():
                ## merge the OTU's taxonomy with OTL information
                res = taxonomy + operational_taxon_list_df[operational_taxon_list_df['Class'].str.contains(taxonomy[2])][['DV-NR.', 'Taxon']].values.tolist()[0]

            ## test phylum
            elif taxonomy[1] != '' and taxonomy[1] in operational_taxon_list_df['Phylum'].values.tolist():
                ## merge the OTU's taxonomy with OTL information
                res = taxonomy + operational_taxon_list_df[operational_taxon_list_df['Phylum'].str.contains(taxonomy[1])][['DV-NR.', 'Taxon']].values.tolist()[0]
            else:
                res = taxonomy + ['', '']
                dropped_list.append(taxonomy)

            hit_list.append(res)

        ## create a dataframe
        ## create a new df and export it as TaXon table
        df = pd.DataFrame(hit_list, columns=TaXon_table_df.columns[0:7].values.tolist() + ['DV-NR.', 'Taxon (Phylib)'])
        concatenated_df = pd.concat([df, TaXon_table_df[samples_list + ['Similarity', 'Status', 'seq']]], axis=1)
        reordered_df = concatenated_df[['ID', 'Phylum', 'Class', 'Order', 'Family', 'Genus', 'Species', 'Similarity', 'Status', 'DV-NR.', 'Taxon (Phylib)', 'seq'] + samples_list]
        reordered_metadata_df = add_metadata(reordered_df, meta_data_df)
        phylib_TaXon_Table = Path(str(path_to_outdirs) + "/" + "TaXon_tables" + "/" + TaXon_table_xlsx.stem + "_phylib.xlsx")
        reordered_metadata_df.to_excel(phylib_TaXon_Table, index=False)

        # create an output list for phylib
        messwerte_list = []
        messtellen_list = []

        for sample in samples_list:
            ## collect information about the sampling site
            sample_metadata_df = Meta_data_table_df.loc[Meta_data_table_df['Samples'] == sample]
            messstelle = sample
            ökoregion = sample_metadata_df['Ökoregion'].values.tolist()[0]
            Makrophytenverödung = sample_metadata_df['Makrophytenverödung'].values.tolist()[0]
            Begründung = sample_metadata_df['Begründung'].values.tolist()[0]
            Helophytendominanz = sample_metadata_df['Helophytendominanz'].values.tolist()[0]
            Diatomeentyp = sample_metadata_df['Diatomeentyp'].values.tolist()[0]
            Phytobenthostyp = sample_metadata_df['Phytobenthostyp'].values.tolist()[0]
            Makrophytentyp = sample_metadata_df['Makrophytentyp'].values.tolist()[0]
            WRRL_Typ = sample_metadata_df['WRRL-Typ'].values.tolist()[0]
            Gesamtdeckungsgrad = sample_metadata_df['Gesamtdeckungsgrad'].values.tolist()[0]
            messtellen_list.append([messstelle, ökoregion, Makrophytenverödung, Begründung, Helophytendominanz, Diatomeentyp, Phytobenthostyp, Makrophytentyp, WRRL_Typ, Gesamtdeckungsgrad])

            ## calcualte sum of reads for taxa with multiple OTUs
            sample_df = reordered_metadata_df[[sample, 'DV-NR.', 'Taxon (Phylib)']]
            reads_dict = {}
            for taxon in sample_df.values.tolist():
                key = taxon[2]
                values = taxon[:2]
                if key != '':
                    if key not in reads_dict.keys():
                        reads_dict[key] = values
                    else:
                        reads_dict[key] = [reads_dict[key][0] + values[0], values[1]]

            ## remove duplicates
            samples_taxa = [[key]+values for key,values in reads_dict.items() if values[0] != 0]

            ## if PA data is required: Convert to 1/0
            if presence_absence == True:
                samples_taxa = [[i[0], 1, i[2]] for i in samples_taxa]

            ## calculate the overall number of reads/specimens. This is required for later relative abundance calculation
            sum_measurement = sum([i[1] for i in samples_taxa])

            ## loop through all taxa
            for taxon in samples_taxa:
                ## if the taxon (assigned to the OTU) is present in the sample and present on the OTL, continue
                ## add all relevant information to list (for df)
                probe = sample
                taxon_id = taxon[2]
                taxonname = taxon[0]
                form = "o.A."
                messwert = taxon[1] / sum_measurement * 100
                einheit = "%"
                cf = ""
                messwerte_list.append([messstelle, probe, taxon_id, taxonname, form, messwert, einheit, cf])

        phylib_df_1 = pd.DataFrame(messtellen_list, columns=["Messstelle", "Ökoregion", "Makrophytenverödung", "Begründung", "Helophytendominanz", "Diatomeentyp", "Phytobenthostyp", "Makrophytentyp", "WRRL-Typ", "Gesamtdeckungsgrad"])
        phylib_df_2 = pd.DataFrame(messwerte_list, columns=["Messstelle", "Probe", "Taxon", "Taxonname", "Form", "Messwert", "Einheit", "cf"])

        if presence_absence == False:
            phylib_directory = Path(str(path_to_outdirs) + "/" + "Phylib" + "/" + TaXon_table_xlsx.stem)
            phylib_xlsx = Path(str(phylib_directory) + "_phylib_ABUNDANCE.xlsx")
            writer = pd.ExcelWriter(phylib_xlsx, engine='xlsxwriter')
        else:
            phylib_directory = Path(str(path_to_outdirs) + "/" + "Phylib" + "/" + TaXon_table_xlsx.stem)
            phylib_xlsx = Path(str(phylib_directory) + "_phylib_PA.xlsx")
            writer = pd.ExcelWriter(phylib_xlsx, engine='xlsxwriter')
        phylib_df_1.to_excel(writer, sheet_name='Messstelle', index=False)
        phylib_df_2.to_excel(writer, sheet_name='Messwerte', index=False)
        writer.save()

        ################################################################################################################
        ## create some plots and provide statistics
        unique_original_taxa = list(k for k, _ in itertools.groupby(reordered_metadata_df[['Phylum', 'Class', 'Order', 'Family', 'Genus', 'Species']].values.tolist()))
        len(unique_original_taxa)

        unique_OTL_taxa = [i for i in set(reordered_metadata_df['Taxon (Phylib)'].values.tolist()) if i != '']
        len(unique_OTL_taxa)

        phylib_conversion_loss = {}
        for taxon in unique_OTL_taxa:
            all_taxa = reordered_metadata_df[reordered_metadata_df['Taxon (Phylib)'].str.contains(taxon)][['Phylum', 'Class', 'Order', 'Family', 'Genus', 'Species']].values.tolist()
            unique_taxa = set([''.join(i) for i in all_taxa])
            phylib_conversion_loss[taxon] = len(unique_taxa)

        perlodes_conversion_loss = {k: v for k, v in sorted(phylib_conversion_loss.items(), key=lambda item: item[1])}

        fig = go.Figure()
        x_values = [i[:19] for i in list(perlodes_conversion_loss.keys())[-30:][::-1]] ## [:20] to cut names off
        y_values = list(perlodes_conversion_loss.values())[-30:][::-1]
        fig.add_trace(go.Bar(x=x_values, y=y_values, marker_color='blue'))
        fig.update_yaxes(title='# taxa')
        fig.update_layout(title='', showlegend=False, font_size=14, width=width_value, height=height_value, template=template)
        phylib_plot = Path(str(phylib_directory) + "_phylib.pdf")
        fig.write_image(phylib_plot)
        phylib_plot = Path(str(phylib_directory) + "_phylib.html")
        fig.write_html(phylib_plot)

def convert_to_diathor(path_to_outdirs, taxon_table_xlsx, taxon_table_df, samples, metadata_df, traits_df, tool_settings):
    # Make copies of input dataframes to prevent altering the originals
    taxon_table_df = taxon_table_df.copy()

    # Extract relevant settings from the tool settings
    presence_absence = tool_settings['presence_absence']
    taxon_table_taxonomy = taxon_table_df.columns.tolist()[1:7]

    all_reads = []
    for sample in samples:
        total_reads = taxon_table_df[sample].sum()
        taxon_dict = {}
        for OTU in taxon_table_df[taxon_table_taxonomy + [sample]].values.tolist():
            taxon = [item for item in OTU[0:6] if item != ''][-1]
            if taxon not in taxon_dict.keys():
                n_reads = OTU[-1]
                taxon_dict[taxon] = n_reads
            else:
                n_reads = OTU[-1]
                taxon_dict[taxon] = taxon_dict[taxon] + n_reads
        if presence_absence == True:
            pa_list = [1 if i != 0 else 0 for i in taxon_dict.values()]
            all_reads.append(pa_list)
        elif presence_absence == 'Relative':
            rel_list = [round(i / total_reads * 100, 2) for i in taxon_dict.values()]
            all_reads.append(rel_list)
        else:
            all_reads.append(list(taxon_dict.values()))

    index = [i for i in range(1,1+len(taxon_dict.keys()))]
    species = list(taxon_dict.keys())

    diathor_df = pd.DataFrame(species, columns=['species'])
    diathor_df.insert(0, '', index)
    diathor_df = pd.concat([diathor_df, pd.DataFrame(all_reads, index=samples).transpose()], axis=1)

    # write the filtered list to a dataframe
    if presence_absence == True:
        diathor_directory = Path(str(path_to_outdirs) + "/" + "Diathor" + "/" + taxon_table_xlsx.stem)
        diathor_csv = Path(str(diathor_directory) + "_DiaThor_PA.csv")
        diathor_df.to_csv(diathor_csv, index=False, sep=',')
    elif presence_absence == 'Relative':
        diathor_directory = Path(str(path_to_outdirs) + "/" + "Diathor" + "/" + taxon_table_xlsx.stem)
        diathor_csv = Path(str(diathor_directory) + "_DiaThor_RELATIVE_ABUNDANCE.csv")
        diathor_df.to_csv(diathor_csv, index=False, sep=',')
    else:
        diathor_directory = Path(str(path_to_outdirs) + "/" + "DiaThor" + "/" + taxon_table_xlsx.stem)
        diathor_csv = Path(str(diathor_directory) + "_DiaThor_ABUNDANCE.csv")
        diathor_df.to_csv(diathor_csv, index=False, sep=',')

    st.success(f'Wrote Diathor input file to: {diathor_csv}')

    ## Calculate Diathor
    output_folder = diathor_csv.parent.joinpath(diathor_csv.stem)
    os.makedirs(output_folder, exist_ok=True)
    output_xlsx = output_folder.joinpath(str(diathor_csv.name).replace('.csv', '.xlsx'))
    log_txt = Path(str(output_xlsx).replace('.xlsx', '.log'))
    f = open(log_txt, 'w')
    script_dir = os.path.dirname(os.path.abspath(__file__))
    script_path = Path(os.path.join(script_dir, 'Rscripts', 'Diathor.R'))
    subprocess.call(['Rscript', script_path, '-i', diathor_csv, '-o', output_xlsx], stdout=f)
    f.close()
    st.success(f'Wrote Diathor calculations to: {output_xlsx}')

def convert_to_efi(path_to_outdirs, taxon_table_xlsx, taxon_table_df, samples, metadata_df, traits_df, tool_settings):
    # Make copies of input dataframes to prevent altering the originals
    taxon_table_df = taxon_table_df.copy()
    metadata_df = metadata_df.copy()

    # Extract relevant settings from the tool settings
    presence_absence = tool_settings['presence_absence']
    taxon_table_taxonomy = taxon_table_df.columns.tolist()[1:7]

    # Prepare metadata
    efi_metadata_df = metadata_df[['Sample'] + tool_settings['efi_columns']]

    # Filter samples
    samples = [sample for sample in samples if '' not in efi_metadata_df[efi_metadata_df['Sample'] == sample].values.tolist()[0]]

    if len(samples) == 0:
        st.warning('Please fill out the required metadata for at least one sample!')
        return

    # collect species
    i = 1
    efi_df_values = []
    for sample in samples:
        taxon_dict = {}
        total_reads = taxon_table_df[sample].sum()
        for OTU in taxon_table_df[taxon_table_taxonomy + [sample]].values.tolist():
            taxon = [item for item in OTU[0:6] if item != ''][-1]
            if taxon not in taxon_dict.keys():
                n_reads = OTU[-1]
                taxon_dict[taxon] = n_reads
            else:
                n_reads = OTU[-1]
                taxon_dict[taxon] = taxon_dict[taxon] + n_reads
        efi_sample_metadata = efi_metadata_df[efi_metadata_df['Sample'] == sample].values.tolist()
        for taxon, reads in taxon_dict.items():
            if reads != 0:
                if presence_absence == True:
                    total_number_run1 = 2
                    number_length_below_150 = 1
                    number_length_above_150 = 1
                elif presence_absence == 'Relative':
                    total_number_run1 = math.ceil(reads / total_reads * 100)
                    number_length_below_150 = math.ceil(total_number_run1 / 2)
                    number_length_above_150 = math.ceil(total_number_run1 / 2)
                    total_number_run1 = number_length_above_150 * 2 # needs to be updated to fit the sum of both
                else:
                    number_length_below_150 = math.ceil(reads/2)
                    number_length_above_150 = math.ceil(reads/2)
                    total_number_run1 = number_length_above_150 * 2 # needs to be updated to fit the sum of both
                species_values = efi_sample_metadata[0] + [0, taxon, total_number_run1, number_length_below_150, number_length_above_150, 'abc', i, taxon]
                efi_df_values.append(species_values)
                i +=1

    # construct EFI dataframe
    efi_df = pd.DataFrame(efi_df_values, columns=['Sample.code'] + [i.replace('EFI_', '') for i in tool_settings['efi_columns']] + ['Medit', 'Species', 'Total.number.run1', 'Number.length.below.150', 'Number.length.over.150', 'Sampling.location', 'code', 'species'])

    # add dummy values
    # other EFI will crash in some cases...
    efi_table = Path(os.path.join(st.session_state['script_dir'], 'WFD_conversion', 'efi_table.xlsx'))
    efi_dummy_df = pd.read_excel(efi_table, sheet_name='Sheet3').fillna('')
    efi_dummy_df = efi_dummy_df[efi_df.columns.tolist()]
    efi_df = pd.concat([efi_df, efi_dummy_df], ignore_index=True)
    efi_df['code'] = [i for i in range(1, len(efi_df)+1)]

    # write the filtered list to a dataframe
    if presence_absence == True:
        efi_directory = Path(str(path_to_outdirs) + "/" + "EFI" + "/" + taxon_table_xlsx.stem)
        efi_xlsx = Path(str(efi_directory) + "_EFI_PA.xlsx")
        efi_df.to_excel(efi_xlsx, index=False)
    elif presence_absence == 'Relative':
        efi_directory = Path(str(path_to_outdirs) + "/" + "EFI" + "/" + taxon_table_xlsx.stem)
        efi_xlsx = Path(str(efi_directory) + "_EFI_RELATIVE_ABUNDANCE.xlsx")
        efi_df.to_excel(efi_xlsx, index=False)
    else:
        efi_directory = Path(str(path_to_outdirs) + "/" + "EFI" + "/" + taxon_table_xlsx.stem)
        efi_xlsx = Path(str(efi_directory) + "_EFI_ABUNDANCE.xlsx")
        efi_df.to_excel(efi_xlsx, index=False)

    st.success(f'Wrote EFI input file to: {efi_xlsx}')

    ## Calculate EFI
    output_folder = efi_xlsx.parent.joinpath(efi_xlsx.stem)
    os.makedirs(output_folder, exist_ok=True)
    output_xlsx = output_folder.joinpath(str(efi_xlsx.name).replace('.xlsx', '_EFI.xlsx'))
    log_txt = Path(str(output_xlsx).replace('.xlsx', '.log'))
    f = open(log_txt, 'w')
    script_dir = os.path.dirname(os.path.abspath(__file__))
    script_path = Path(os.path.join(script_dir, 'Rscripts', 'EFI.R'))
    subprocess.call(['Rscript', script_path, '-i', efi_xlsx, '-o', output_xlsx], stdout=f)
    f.close()
    st.success(f'Wrote EFI calculations to: {output_xlsx}')




import streamlit as st
import pandas as pd
import numpy as np
import subprocess
import glob, sys, time, statistics, os.path, random
from pathlib import Path
import psutil
import plotly.graph_objects as go
from taxontabletools2.start import TTT_variables
from taxontabletools2.utilities import load_df
from taxontabletools2.utilities import collect_replicates
from taxontabletools2.utilities import collect_sample_stats
from taxontabletools2.basic_stats import basic_stats_reads
from taxontabletools2.basic_stats import basic_stats_OTUs
from taxontabletools2.basic_stats import taxonomic_resolution
from taxontabletools2.basic_stats import taxonomic_richness
from taxontabletools2.utilities import collect_traits
from taxontabletools2.utilities import strip_traits
from taxontabletools2.utilities import check_replicates
from taxontabletools2.table_processing import replicate_merging
from taxontabletools2.table_processing import negative_control_subtraction
from taxontabletools2.table_processing import read_based_filter
from taxontabletools2.table_processing import read_based_normalisation
from taxontabletools2.table_processing import taxonomic_filtering
from taxontabletools2.table_processing import sample_filtering
from taxontabletools2.start import start

# Call the sidebar function
settings = start()
users_settings = settings['new_user_preferences_dict']

## Page title
st.write(""" ## Taxon Table conversion """)

def merge_otu_and_taxonomy_tables():
    """
    Merges an OTU table and a taxonomy table based on unique IDs.
    Ensures proper formatting, column alignment, and metadata creation.
    """

    # Load OTU table
    otu_table = pd.read_excel(st.session_state['otu_table_path']).fillna('')

    # Determine the correct sheet name for the taxonomy table
    taxonomy_sheets = {'APSCALE': 'Taxonomy table', 'BOLDigger': 'Sheet1'}
    sheet_name = taxonomy_sheets.get(st.session_state['taxonomy_table_format'], None)

    if sheet_name is None:
        st.error('Invalid taxonomy table format.')
        return

    # Load taxonomy table
    try:
        taxonomy_table = pd.read_excel(st.session_state['taxonomy_table_path'], sheet_name=sheet_name).fillna('')
    except ValueError:
        st.error(f"Worksheet named '{sheet_name}' not found")
        return

    # Extract unique IDs from both tables
    otu_table_IDs = otu_table['unique_ID'].tolist()
    taxonomy_column_map = {'APSCALE': 'unique ID', 'BOLDigger': 'id'}
    taxonomy_id_col = taxonomy_column_map.get(st.session_state['taxonomy_table_format'])

    if taxonomy_id_col in taxonomy_table.columns:
        taxonomy_table_IDs = taxonomy_table[taxonomy_id_col].tolist()
        taxonomy_table.rename(columns={taxonomy_id_col: 'unique_ID'}, inplace=True)

        # Additional renaming for BOLDigger format
        if st.session_state['taxonomy_table_format'] == 'BOLDigger':
            taxonomy_table.rename(columns={'pct_identity': 'Similarity'}, inplace=True)

        cont = True
    else:
        cont = False

    # Check for ID mismatches
    if otu_table_IDs != taxonomy_table_IDs:
        st.error(
            'WARNING: The IDs of the Read table and Taxonomy table do not match! Make sure they are identically sorted!')
        return
    elif not cont:
        st.error('WARNING: Unable to find the required sheets in the provided taxonomy table.')
        return

    # Merge OTU and taxonomy tables on unique_ID
    taxon_table = taxonomy_table.merge(otu_table, how='inner', on='unique_ID')

    # Move 'Seq' column to correct position
    first_sample = otu_table.columns[2]
    loc_first_sample = taxon_table.columns.get_loc(first_sample)
    seq = taxon_table.pop('Seq')  # Remove 'Seq' column
    taxon_table.insert(loc_first_sample, 'Seq', seq)  # Insert at new position

    # Rename 'unique_ID' to 'ID'
    taxon_table.rename(columns={'unique_ID': 'ID'}, inplace=True)

    # Create metadata DataFrame
    samples = otu_table.columns[2:-1]
    metadata_df = pd.DataFrame({'Sample': samples, 'Metadata': ''})

    # Ensure 'Kingdom' column exists
    if 'Kingdom' not in taxon_table.columns:
        loc_phylum = taxon_table.columns.get_loc('Phylum')
        taxon_table.insert(loc_phylum, 'Kingdom', 'Life')

    # Merge 'Genus' and 'Species' columns if required
    if st.session_state['genus_species_merge'] == 'Merge':
        taxon_table['Species'] = taxon_table.apply(
            lambda row: f"{row['Genus']} {row['Species']}" if row['Genus'] and row['Species'] else row['Species'],
            axis=1
        )

    # Display merged table
    st.success('Successfully merged your tables:')
    st.dataframe(taxon_table)

    # Save merged table and metadata to an Excel file
    with pd.ExcelWriter(st.session_state['taxon_table_path']) as writer:
        taxon_table.to_excel(writer, sheet_name='Taxon Table', index=False)
        metadata_df.to_excel(writer, sheet_name='Metadata Table', index=False)

    st.success(f'Saved as "{st.session_state["taxon_table_path"]}"')

a1, a2 = st.columns(2)
with a1:
    st.session_state['otu_table_path'] = st.file_uploader('Select your Read table:')
    st.session_state['otu_table_format'] = st.selectbox('Read table format', ['APSCALE'])

    if st.session_state['otu_table_path'] == None:
        st.write('Please select a Read table!')

with a2:
    st.session_state['taxonomy_table_path'] = st.file_uploader('Select your Taxonomy table:')
    st.session_state['taxonomy_table_format'] = st.selectbox('Taxonomy table format', ['APSCALE', 'BOLDigger'])
    st.session_state['genus_species_merge'] = st.selectbox('Merge the genus and species epithet column', ['Merge', 'No merge'], index=1)

    if st.session_state['taxonomy_table_path'] == None:
        st.write('Please select a Taxonomy table!')

## Save table to project folder
st.session_state['taxon_table_name'] = st.text_input('Taxon table name',
                                                     f'{st.session_state["path_to_outdirs"].name}_taxon_table') + '.xlsx'
st.session_state['taxon_table_path'] = st.session_state['path_to_outdirs'].joinpath('TaXon_tables',
                                                                                  st.session_state['taxon_table_name'])

if st.session_state['taxonomy_table_path'] != None and st.session_state['otu_table_path'] != None:
    if st.button('Convert to TaXon table format'):
        merge_otu_and_taxonomy_tables()


















import webbrowser
import streamlit as st
import pandas as pd
import numpy as np
import subprocess
import glob, sys, time, statistics, os.path, random, os
from pathlib import Path
import psutil
import plotly.graph_objects as go
from taxontabletools2.start import TTT_variables
from taxontabletools2.utilities import load_df
from taxontabletools2.utilities import collect_replicates
from taxontabletools2.utilities import collect_sample_stats
from taxontabletools2.basic_stats import basic_stats_reads
from taxontabletools2.basic_stats import basic_stats_OTUs
from taxontabletools2.basic_stats import taxonomic_resolution
from taxontabletools2.basic_stats import taxonomic_richness
from taxontabletools2.utilities import collect_traits
from taxontabletools2.utilities import strip_traits
from taxontabletools2.utilities import check_replicates
from taxontabletools2.utilities import update_taxon_table
from taxontabletools2.table_processing import replicate_merging
from taxontabletools2.table_processing import negative_control_subtraction
from taxontabletools2.table_processing import read_based_filter
from taxontabletools2.table_processing import read_based_normalisation
from taxontabletools2.table_processing import taxonomic_filtering
from taxontabletools2.table_processing import sample_filtering
from taxontabletools2.WFD_tools import convert_to_perlodes
from taxontabletools2.WFD_tools import convert_to_phylib
from taxontabletools2.WFD_tools import convert_to_diathor
from taxontabletools2.WFD_tools import convert_to_efi
import streamlit as st
from taxontabletools2.start import start

# Call the sidebar function
settings = start()
users_settings = settings['new_user_preferences_dict']

## Page title
st.write(""" ## WFD conversion """)
st.markdown("---")

# Ensure the session state variables are retained across subpages
if 'TaXon_table_df' not in st.session_state:
    error1, error2, error3 = st.columns(3)
    with error2:
        st.write('##### Please load a TaXon table to continue!')

elif 'TaXon_table_xlsx' not in st.session_state:
    error1, error2, error3 = st.columns(3)
    with error2:
        st.write('##### Please load a TaXon table to continue!')

elif st.session_state['TaXon_table_df'].empty == True:
    error1, error2, error3 = st.columns(3)
    with error2:
        st.write('##### Please check your TaXon table. It seems to be empty!')

elif st.session_state['TaXon_table_xlsx'].is_file() == False:
    error1, error2, error3 = st.columns(3)
    with error2:
        st.write('##### The TaXon table does not exist in the selected project folder!')

########################################################################################################################

else:

    ####################################################################################################################
    st.write('### 1) Macroinvertebrates')
    st.write('### Perlodes (Water Framework Directive Germany)')
    perlodes1, perlodes2 = st.columns(2)
    with perlodes1:
        presence_absence_perlodes = st.selectbox('Convert to presence/absence data:', [True, False, 'Relative'], index=0, key='perlodes2')
    with perlodes2:
        if st.button('Visit the Perlodes home page to calculate status classes'):
            webbrowser.open('https://gewaesser-bewertung-berechnung.de/index.php/login.html')

    available_metadata_perlodes = st.session_state['metadata_df'].columns.tolist()[1:]
    perlodes_columns = ["Perlodes_river_type", 'Perlodes_taxa_list', 'Perlodes_usage']

    if set(perlodes_columns).issubset(set(available_metadata_perlodes)):
        st.success('All required columns to create Perlodes input file were found!')
        if st.button('Convert to Perlodes input'):
            convert_to_perlodes(st.session_state['path_to_outdirs'],
                                st.session_state['TaXon_table_xlsx'],
                                st.session_state['TaXon_table_df'],
                                st.session_state['samples'],
                                st.session_state['metadata_df'],
                                st.session_state['traits_df'],
                                {'presence_absence': presence_absence_perlodes}
                                )
    else:
        st.warning('Not all required columns are present in the metadata!')
        if st.button('Add Perlodes columns to to table'):
            for col in perlodes_columns:
                st.session_state['metadata_df'][col] = ''
            update_taxon_table(
                st.session_state['TaXon_table_xlsx'],
                st.session_state['TaXon_table_df'],
                st.session_state['traits_df'],
                st.session_state['metadata_df'],
                '')
            st.success('Required columns were added to table. Please fill out the respective Perlodes columns and reload the table!')

    with st.expander("Perlodes variables"):
        st.write('Coming soon')

    ####################################################################################################################
    st.markdown("---")
    st.write('### 2) Diatoms')
    st.write('### Phylib (Water Framework Directive Germany)')
    phylib1, phylib2 = st.columns(2)
    with phylib1:
        presence_absence_phylib = st.selectbox('Convert to presence/absence data:', [True, False, 'Relative'], index=0, key='phylib2')
    with phylib2:
        if st.button('Visit the Phylib home page to calculate status classes'):
            webbrowser.open('https://gewaesser-bewertung-berechnung.de/index.php/login.html')
    if st.button(f'Create phylib upload file.'):
        print('Coming soon!')

    with st.expander("Phylib variables"):
        st.write('Coming soon')

    st.write('### Diathor (Multiple diatom indices)')
    diathor1, diathor2 = st.columns(2)
    with diathor1:
        presence_absence_diathor = st.selectbox('Convert to presence/absence data:', [True, False, 'Relative'], index=0, key='diathor2')
    with diathor2:
        pass
    if st.button(f'Calculate diatom indices with Diathor.'):
        convert_to_diathor(st.session_state['path_to_outdirs'],
                           st.session_state['TaXon_table_xlsx'],
                           st.session_state['TaXon_table_df'],
                           st.session_state['samples'],
                           st.session_state['metadata_df'],
                           st.session_state['traits_df'],
                           {'presence_absence':presence_absence_diathor}
                           )

    with st.expander("Diathor variables"):
        st.write('Coming soon')

    ####################################################################################################################
    st.markdown("---")
    st.write('### 3) Fish')
    st.write('### EFI (European Fish Index)')
    efi1, efi2 = st.columns(2)
    with efi1:
        presence_absence_efi = st.selectbox('Convert to presence/absence data:', [True, False, 'Relative'], index=0, key='efi2')
    with efi2:
        pass

    available_metadata_efi = st.session_state['metadata_df'].columns.tolist()[1:]
    efi_columns = [
                        "EFI_Day", "EFI_Month", "EFI_Year", "EFI_Longitude", "EFI_Latitude",
                        "EFI_Actual.river.slope", "EFI_Temp.jul", "EFI_Temp.jan", "EFI_Floodplain.site",
                        "EFI_Water.source.type", "EFI_Geomorph.river.type", "EFI_Distance.from.source",
                        "EFI_Area.ctch", "EFI_Natural.sediment", "EFI_Ecoreg", "EFI_Eft.type",
                        "EFI_Fished.area", "EFI_Method"
                    ]
    if set(efi_columns).issubset(set(available_metadata_efi)):
        st.success('All required columns to calculate the EFI were found!')
        if st.button('Calculate EFI'):
            convert_to_efi(st.session_state['path_to_outdirs'],
                               st.session_state['TaXon_table_xlsx'],
                               st.session_state['TaXon_table_df'],
                               st.session_state['samples'],
                               st.session_state['metadata_df'],
                               st.session_state['traits_df'],
                               {'efi_columns': efi_columns, 'presence_absence': presence_absence_efi}
                               )

    else:
        st.warning('Not all required columns are present in the metadata!')
        if st.button('Add EFI columns to to table'):
            for col in efi_columns:
                st.session_state['metadata_df'][col] = ''
            update_taxon_table(
                st.session_state['TaXon_table_xlsx'],
                st.session_state['TaXon_table_df'],
                st.session_state['traits_df'],
                st.session_state['metadata_df'],
                '')
            st.success('Required columns were added to table. Please fill out the respective EFI columns and reload the table!')

    with st.expander("EFI variables"):
        efi_table = Path(os.path.join(st.session_state['script_dir'], 'WFD_conversion', 'efi_table.xlsx'))
        st.table(pd.read_excel(efi_table, sheet_name='Sheet1').fillna(''))
        st.table(pd.read_excel(efi_table, sheet_name='Sheet2').fillna(''))
        efi_map = Path(os.path.join(st.session_state['script_dir'], 'WFD_conversion', 'efi_ecoregions.png'))
        caption = 'Map of Illies ecoregions represented in EFI+ and the additional Mediterranean region. Source: EFI manual'
        st.image(str(efi_map), caption=caption, use_column_width=True)


from taxontabletools2.utilities import load_df, collect_traits, strip_traits
from pathlib import Path
import pandas as pd

# DiATHOR TEST
path_to_outdirs = Path('/Users/tillmacher/Desktop/TTT_projects/AA_test')
taxon_table_xlsx = Path('/Users/tillmacher/Desktop/TTT_projects/AA_test/TaXon_tables/LL_diatoms_taxon_table_cons_NCsub_merged_diatoms.xlsx')
taxon_table_df = load_df(taxon_table_xlsx)
selected_metadata = 'River type'
traits_df = collect_traits(taxon_table_df)
taxon_table_df = strip_traits(taxon_table_df)
samples = list(taxon_table_df.columns[9:])
metadata_df = pd.read_excel(taxon_table_xlsx, sheet_name='Metadata Table').fillna('')
user_settings = {'project_name': 'AA_test', 'path_to_outdirs': Path('/Users/tillmacher/Desktop/TTT_projects'),
                 'plot_height': 1000, 'plot_width': 1000, 'show_legend': True, 'template': 'simple_white',
                 'font_size': 20, 'clustering_unit': 'ESVs', 'scatter_size': 15, 'color_1': 'navy', 'color_2': 'teal',
                 'colorsequence': 'Plotly', 'colorscale': 'blues'}
users_settings = user_settings
tool_settings = {'selected_metadata': 'River type', 'presence_absence': True}
perlodes_TTT_conversion_xlsx = Path('/Users/tillmacher/Documents/GitHub/TaxonTableTools2.0/taxontabletools2/WFD_conversion/perlodes_TTT_conversion.xlsx')
script_dir = "/Users/tillmacher/Applications/miniconda3/miniconda3/envs/TTT/lib/python3.12/site-packages/taxontabletools2"
script_path = Path(os.path.join(script_dir, 'Rscripts',  'Diathor.R'))





# EFI TEST
path_to_outdirs = Path('/Users/tillmacher/Desktop/TTT_projects/AA_test')
taxon_table_xlsx = Path('/Users/tillmacher/Desktop/TTT_projects/AA_test/TaXon_tables/LL_vertebrates_taxon_table_cons_NCsub_merged_vertebrates_fish.xlsx')
taxon_table_df = load_df(taxon_table_xlsx)
traits_df = collect_traits(taxon_table_df)
taxon_table_df = strip_traits(taxon_table_df)
samples = list(taxon_table_df.columns[9:])
metadata_df = pd.read_excel(taxon_table_xlsx, sheet_name='Metadata Table').fillna('')
user_settings = {'project_name': 'AA_test', 'path_to_outdirs': Path('/Users/tillmacher/Desktop/TTT_projects'),
                 'plot_height': 1000, 'plot_width': 1000, 'show_legend': True, 'template': 'simple_white',
                 'font_size': 20, 'clustering_unit': 'ESVs', 'scatter_size': 15, 'color_1': 'navy', 'color_2': 'teal',
                 'colorsequence': 'Plotly', 'colorscale': 'blues'}
users_settings = user_settings
efi_columns = [
    "EFI_Day", "EFI_Month", "EFI_Year", "EFI_Longitude", "EFI_Latitude",
    "EFI_Actual.river.slope", "EFI_Temp.jul", "EFI_Temp.jan", "EFI_Floodplain.site",
    "EFI_Water.source.type", "EFI_Geomorph.river.type", "EFI_Distance.from.source",
    "EFI_Area.ctch", "EFI_Natural.sediment", "EFI_Ecoreg", "EFI_Eft.type",
    "EFI_Fished.area", "EFI_Method"
]
tool_settings = {'efi_columns': efi_columns, 'presence_absence': True}
script_dir = "/Users/tillmacher/Applications/miniconda3/miniconda3/envs/TTT/lib/python3.12/site-packages/taxontabletools2"
script_path = Path(os.path.join(script_dir, 'Rscripts',  'EFI.R'))






# PERLODES TEST
path_to_outdirs = Path('/Users/tillmacher/Desktop/TTT_projects/AA_test')
taxon_table_xlsx = Path('/Users/tillmacher/Desktop/TTT_projects/AA_test/TaXon_tables/LL_invertebrates_taxon_table_cons_NCsub_merged_invertebrates_aquatic.xlsx')
taxon_table_df = load_df(taxon_table_xlsx)
selected_metadata = 'River type'
traits_df = collect_traits(taxon_table_df)
taxon_table_df = strip_traits(taxon_table_df)
samples = list(taxon_table_df.columns[9:])
metadata_df = pd.read_excel(taxon_table_xlsx, sheet_name='Metadata Table').fillna('')
user_settings = {'project_name': 'AA_test', 'path_to_outdirs': Path('/Users/tillmacher/Desktop/TTT_projects'),
                 'plot_height': 1000, 'plot_width': 1000, 'show_legend': True, 'template': 'simple_white',
                 'font_size': 20, 'clustering_unit': 'ESVs', 'scatter_size': 15, 'color_1': 'navy', 'color_2': 'teal',
                 'colorsequence': 'Plotly', 'colorscale': 'blues'}
users_settings = user_settings
tool_settings = {'selected_metadata': 'River type', 'presence_absence': True}




















